use std::collections::HashMap;

use dbus::arg::messageitem::MessageItem::Dict;
use dbus::arg::messageitem::{MessageItem, MessageItemDict};
use dbus::Message;

#[derive(Debug, PartialEq, Copy, Clone)]
pub enum PlayerStatus {
    Playing,
    Paused,
    Stopped,
    Unknown,
}

impl From<String> for PlayerStatus {
    fn from(str: String) -> Self {
        match str.as_str() {
            "Playing" => PlayerStatus::Playing,
            "Paused" => PlayerStatus::Paused,
            "Stopped" => PlayerStatus::Stopped,
            _ => PlayerStatus::Unknown,
        }
    }
}

impl From<&MessageItemDict> for PlayerStatus {
    fn from(message_dict: &MessageItemDict) -> Self {
        let vec_hash_map = message_dict.clone().into_vec();
        let mut message_map = HashMap::new();
        for (key_item, val_item) in vec_hash_map.into_iter() {
            if let MessageItem::Str(key) = key_item {
                message_map.insert(key, val_item);
            }
        }
        if let Some(status_variant) = message_map.get("PlaybackStatus") {
            debug!("Current playback status: {:?}", status_variant.clone());
            match status_variant {
                MessageItem::Variant(variant) => {
                    if let MessageItem::Str(val) = *variant.clone() {
                        return PlayerStatus::from(val);
                    }
                    PlayerStatus::Unknown
                }
                _ => PlayerStatus::Unknown,
            }
        } else {
            PlayerStatus::Unknown
        }
    }
}

impl From<&Message> for PlayerStatus {
    fn from(message: &Message) -> Self {
        let items = message.get_items();
        let arg_dict = items.get(1);
        if arg_dict.is_none() {
            return PlayerStatus::Unknown;
        }
        let arg_dict = arg_dict.unwrap();
        match arg_dict {
            Dict(dict) => Self::from(dict),
            _ => {
                error!("Can't parse playback status");
                error!("Unknown argument type");
                PlayerStatus::Unknown
            }
        }
    }
}
